@@ -11,14 +11,10 @@ at_exit() (
     set +e
 
     [[ -d "$TMPDIR" ]] && rm -rf "$TMPDIR"
-
-    udevadm control --log-level=info
 )
 
 trap at_exit EXIT
 
-udevadm control --log-level=debug
-
 TMPDIR="$(mktemp -d)"
 truncate -s 16M "$TMPDIR"/foo.raw
 mkfs.ext4 "$TMPDIR"/foo.raw
